#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of CloudFS.
#
# CloudFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# CloudFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License *
# along with CloudFS.  If not, see <http://www.gnu.org/licenses/>.

from bottle import route, post, run, view, debug, request, response, \
		   TEMPLATE_PATH

import os
import sys
import socket
import string

import cfs_paths
import cfs_utils
import volmap
import cfs_utils
import cfs_add_node
import cfs_add_directory
import cfs_add_volume
import cfs_rm_volume
import cfs_start_volume
import cfs_stop_volume
import cfs_add_tenant
import cfs_delete_tenant
import cfs_enable_tenant
import cfs_update_cert

@route("/")
@route("/cfg")
@route("/cfgmain")
@view("cfgmain.html")
def cfg_main():
	return dict()

@route("/cluster")
@view("cluster.html")
def show_cluster():
	# TBD: handle glusterd presence/startup check sanely
	cfs_utils.run_cmd("chkconfig","--add glusterd")
	cfs_utils.run_cmd("chkconfig","glusterd on")
	cfs_utils.run_cmd("service","glusterd start")
	node_list = string.join(cfs_utils.get_members(),"<br />")
	cert_path = os.path.join(cfs_paths.info_dir,"server.pem")
	if os.path.isfile(cert_path):
		cert_link='<a href="/cluster/certificate">get certificate</a>'
		cert_para="<p>%s</p><br />" % cert_link
	else:
		cert_para=""
	return dict(node_list=node_list,cert_para=cert_para)

@route("/cluster/certificate")
def show_cert():
	response.content_type = "text/plain"
	cert_path = os.path.join(cfs_paths.info_dir,"server.pem")
	return open(cert_path,"r")

@post("/cluster/add_node")
def add_node():
	return cfs_add_node.run_www()

# TBD: implement remove_node

@route("/volumes")
@view("volumes.html")
def show_volumes():
	brick_list = cfs_utils.get_bricks()
	mount_list = cfs_utils.get_mounts(brick_list)
	node_list = cfs_utils.get_members()
	# TBD: allow adding arbitrary directories instead of just mountpoints
	return dict(bricks=brick_list,mounts=mount_list,nodes=node_list)

@post("/volumes/add_directory")
def add_directory ():
	return cfs_add_directory.run_www()

@post("/volumes/add_dir_local")
def add_dir_local ():
	path = request.forms.get("path")
	return cfs_add_directory.add_local(path)

@post("/volumes/add_volume")
def add_volume():
	return cfs_add_volume.run_www()

@post("/volumes/add_vol_local")
def add_vol_local ():
	vname = request.forms.get("vname")
	use_ssl = request.forms.get("use_ssl")
	return cfs_add_volume.add_local(vname,use_ssl)

@route("volumes/:vol_name/remove")
def rm_volume (vol_name):
	return cfs_rm_volume.run_www(vol_name)

@route("/volumes/:vol_name/rm_local")
def rm_vol_local (vol_name):
	return cfs_rm_volume.rm_local(vol_name)

@route("/volumes/:vol_name/start")
def start_volume(vol_name):
	return cfs_start_volume.run_www(vol_name)

@route("/volumes/:vol_name/start_local")
def start_local (vol_name):
	return cfs_start_volume.start_local(vol_name)

@route("/volumes/:vol_name/stop")
def stop_volume(vol_name):
	return cfs_stop_volume.run_www(vol_name)

@route("/volumes/:vol_name/stop_local")
def stop_local (vol_name):
	return cfs_stop_volume.stop_local(vol_name)

@route("/volumes/:vol_name/tenants")
@view("volume_tenants.html")
def volume_tenants (vol_name):
	db_obj = cfs_utils.open_db()
	tenants = db_obj["vt_"+vol_name].split(",")
	tenants.sort()
	return dict(vol_name=vol_name,tenants=tenants,db=db_obj)

# Used by mount.cloudfs
@route("/:vol_name/fetch")
def fetch_client_vf(vol_name):
	vf_path = "%s/vols/%s/%s-fuse.vol" % (cfs_paths.gfs_dir, vol_name, vol_name)
	return open(vf_path,"r")

# Used by mount.cloudfs
@route("/:vol_name/map")
def map_paths(vol_name):
	return volmap.vol_map(vol_name)

@route("/tenants")
@view("tenants.html")
def show_tenants ():
	db_obj = cfs_utils.open_db()
	return dict(tenants=db_obj)

@post("/tenants/add")
def add_tenant():
	return cfs_add_tenant.run_www()

@post("/tenants/add_local")
def add_tenant_local ():
	tn_name = request.forms.get("tn_name")
	tn_pw = request.forms.get("tn_pw")
	tn_uid_low = request.forms.get("tn_uid_low")
	tn_uid_high = request.forms.get("tn_uid_high")
	tn_gid_low = request.forms.get("tn_gid_low")
	tn_gid_high = request.forms.get("tn_gid_high")
	return cfs_add_tenant.add_local(tn_name,tn_pw,tn_uid_low,tn_uid_high,tn_gid_low,tn_gid_high)

@route("/tenants/:tn_name/delete")
def delete_tenant (tn_name):
	return cfs_delete_tenant.run_www(tn_name)

@route("/tenants/:tn_name/delete_local")
def delete_tenant_local (tn_name):
	return cfs_delete_tenant.delete_local(tn_name)

@route("/tenants/:tn_name/volumes")
@view("tenant_volumes.html")
def show_tenant_volumes (tn_name):
	db_obj = cfs_utils.open_db()
	all_vols = [v[3:] for v in db_obj.keys() if v.startswith("vt_")]
	# print all_vols
	active = db_obj["tv_"+tn_name].split(",")
	active.sort()
	# print active
	return dict(tn_name=tn_name,all_vols=all_vols,active=active)

@post("/tenants/:tn_name/enable")
def enable_tenant_volumes (tn_name):
	return cfs_enable_tenant.run_www(tn_name)

@post("/tenants/enable_local")
def enable_tenant_local ():
	tn_name = request.forms.get("tn_name")
	vol_list = request.forms.get("vol_list")
	return cfs_enable_tenant.enable_local(tn_name,vol_list)

@route("/tenants/:tn_name/show_cert")
@view("tenant_show_cert.html")
def tenant_show_cert (tn_name):
	cert_path = os.path.join(cfs_paths.info_dir,"client_%s.pem"%tn_name)
	if os.path.isfile(cert_path):
		cert_data = open(cert_path).read()
	else:
		cert_data = ""
	return dict(tn_name=tn_name,cert_data=cert_data)

@post("/tenants/:tn_name/update_cert")
def tenant_update_cert (tn_name):
	cert_data = request.forms.get("cert_data")
	return cfs_update_cert.run_www(tn_name,cert_data)

@post("/tenants/set_cert_local")
def tenant_update_cert_local ():
	tn_name = request.forms.get("tn_name")
	cert_data = request.forms.get("cert_data")
	return cfs_update_cert.update_local(tn_name,cert_data)

@route("/styles/:sheet")
def get_style (sheet):
	fqpath = sys.argv[0].rsplit('/', 1)
	return file("%s/styles/%s" % (fqpath[0], sheet), "r")

if __name__ == "__main__":
	debug(True)
	#run(host='0.0.0.0',port=cfs_paths.CLOUDFSD_PORT,server=CherryPyServer)
	fqpath = sys.argv[0].rsplit('/', 1)
	TEMPLATE_PATH.append("%s/views" % fqpath[0])
	run(host='0.0.0.0',port=cfs_paths.CLOUDFSD_PORT)

